<!DOCTYPE kpartgui>
<kpartgui name="konqchoqokplugin" library="konqchoqokplugin" version="4" >
	<MenuBar>
		<Menu name="tools"><text>&amp;Tools</text>
			<Action name="action menu"/>
		</Menu>
	</MenuBar>
</kpartgui>
