<!DOCTYPE kpartgui>
<kpartgui name="choqok" version="17">
    <MenuBar>
        <Menu name="tools"><text>Tools</text>
            <Action name="filterByAuthor" />
	    <Action name="filterByContent" />
        </Menu>
    </MenuBar>
</kpartgui>
